/*
   FNT - nodelist parser
   Main file

   Copyright (C) 2009 Free Software Foundation, Inc.

   Written by:
   Slava Zanko <slavazanko@gmail.com>


   This file is part of the FIDO New Technology.

   FIDO New Technology is free software: you can redistribute it and/or 
   modify it under the terms of the GNU General Public License as published 
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "../include/global.h"
#include "../include/types.h"
#include "../include/error.h"
#include "../include/sql.h"
#include "../include/sql-config.h"
#include "../include/main.h"

#include "main.h"
#include "file_operations.h"
#include "parser.h"

/*** global variables **************************************************/

ndl_t Application;

/*** file scope macro definitions **************************************/

/*** file scope type declarations **************************************/

/*** file scope variables **********************************************/

static const char *config_file = NULL;

/*** file scope functions **********************************************/

static void
print_help (void)
{

    fprintf (stdout,
	     _(PACKAGE "-ndl - nodelist parser - prepare data for FNAR (%s)"),
	     VERSION);

    fprintf (stdout, "\n\nCopyright (C) 2009 Free Software Foundation, Inc.\n\
Written by: Slava Zanko <slavazanko@gmail.com>\n\
This program comes with ABSOLUTELY NO WARRANTY. This is free software, and you\n\
are welcome to redistribute it under certain conditions.\n\n");
    fprintf (stdout, _("Usage:\n"));

    fprintf (stdout, PACKAGE "-ndl -h\n\
" PACKAGE "-ndl [-c <config>] /path/to/nodelist\n\
");

    fprintf (stdout, _("Options are:\n\
   -c      Initial config file to use. Defaults:\n\
                   $FNT_INIT_FILE\n\
                   $HOME/.config/fnt.conf\n\
                   $HOME/.fnt.conf\n\
                   %s/fnt.conf.\n\
   -h              Show this help.\n\n"), SYSCONF_DIR);

    exit (1);
}

static void
get_cmd_parameters (int *argc, char ***argv)
{
    int optchar;

    while ((optchar = getopt (*argc, *argv, "gdc:Li:h?")) >= 0)
    {
	switch (optchar)
	{
	case 'c':
	    config_file = optarg;
	    break;
	case 'h':
	case '?':
	    print_help ();
	    break;
	}
    }
}

static void
check_cmd_parameters (char **argv)
{

}

void
init_db_connects (void)
{
    char *error_str;

    if (!fnt_init ())
    {
	fprintf (stderr, _("ERROR: Unable initialize FNT library\n"));
	exit (2);
    }

    fnt_sql_config_default_cfg_connect =
	fnt_getConfigConnectionFromIniFile (config_file, false);

    fnt_sql_config_default_data_connect =
	fnt_getDataConnectionFromIniFile (config_file, false);


    if (fnt_sql_config_default_cfg_connect == NULL
	|| fnt_sql_config_default_data_connect == NULL)
    {
	error_str = fnt_error_getStackAsString ();
	fprintf (stderr, "%s", error_str);
	g_free (error_str);
	if (fnt_sql_config_default_cfg_connect)
	    fnt_sql_disconnect (&fnt_sql_config_default_cfg_connect);
	if (fnt_sql_config_default_data_connect)
	    fnt_sql_disconnect (&fnt_sql_config_default_data_connect);
	fnt_deinit ();
	exit (2);
    }
}
static void
ndl_deinit (void)
{
    if (fnt_sql_config_default_cfg_connect)
	fnt_sql_disconnect (&fnt_sql_config_default_cfg_connect);
    if (fnt_sql_config_default_data_connect)
	fnt_sql_disconnect (&fnt_sql_config_default_data_connect);

    fnt_deinit ();
}


static void
ndl_process (const char *flname)
{
    int file;

    file = ndl_fileop_openfile (flname);
    if (file == -1)
        ndl_exit(1);

    ndl_parser_run (file);

    ndl_fileop_closefile (file);
}

/*** public functions **************************************************/

void ndl_exit(int return_code)
{
    fnt_error_t *err;

    while ((err = fnt_error_pop ()) != NULL)
    {
	fprintf (stderr, err->errmsg);
	fnt_error_elementFree (err);
    }

    ndl_deinit ();
    exit (return_code);
}


int
main (int argc, char **argv)
{
    setlocale (LC_ALL, "");
    bindtextdomain (PACKAGE, LOCALEDIR);
    textdomain (PACKAGE);

    get_cmd_parameters (&argc, &argv);
    check_cmd_parameters (argv);
    init_db_connects ();

    ndl_process (argv[optind]);

    ndl_deinit ();
    return 0;
}
